@@ -941,17 +941,36 @@ class S3Client {
         // Create canonical headers - must be lowercase and sorted
         $host = parse_url($this->config['endpoint'], PHP_URL_HOST);
         $port = parse_url($this->config['endpoint'], PHP_URL_PORT);
-        
+
         // Include port in host header if it's not standard (80/443)
         if ($port && $port != 80 && $port != 443) {
             $host = $host . ':' . $port;
         }
-        
-        $canonicalHeaders = "content-type:" . $contentType . "\n" .
-                           "host:" . $host . "\n" .
-                           "x-amz-content-sha256:" . $payloadHash . "\n" .
-                           "x-amz-date:" . $timestamp . "\n";
-        $signedHeaders = 'content-type;host;x-amz-content-sha256;x-amz-date';
+
+        // Build canonical headers - MUST include all headers being sent
+        // Headers must be lowercase and sorted alphabetically
+        $canonicalHeaders = "";
+        $signedHeadersList = [];
+
+        // Add content-length if provided (alphabetically before content-type)
+        if ($contentLength !== null) {
+            $canonicalHeaders .= "content-length:" . $contentLength . "\n";
+            $signedHeadersList[] = 'content-length';
+        }
+
+        $canonicalHeaders .= "content-type:" . $contentType . "\n";
+        $signedHeadersList[] = 'content-type';
+
+        $canonicalHeaders .= "host:" . $host . "\n";
+        $signedHeadersList[] = 'host';
+
+        $canonicalHeaders .= "x-amz-content-sha256:" . $payloadHash . "\n";
+        $signedHeadersList[] = 'x-amz-content-sha256';
+
+        $canonicalHeaders .= "x-amz-date:" . $timestamp . "\n";
+        $signedHeadersList[] = 'x-amz-date';
+
+        $signedHeaders = implode(';', $signedHeadersList);
         
         $canonicalRequest = $method . "\n" .
                            $canonicalUri . "\n" .
